import { DatasourceOptions } from '@genesislcap/foundation-comms';
import { ElementStyles } from '@microsoft/fast-element';
import { ErrorObject } from 'ajv';
import { ErrorTranslator } from '@jsonforms/core';
import { Expression } from '@genesislcap/foundation-criteria';
import { FASTElement } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { JsonFormsState } from '@jsonforms/core';
import { JSONSchema7 } from 'json-schema';
import { Logger } from '@genesislcap/foundation-logger';
import { OwnPropsOfControl } from '@jsonforms/core';
import { RankedTester } from '@jsonforms/core';
import { StatePropsOfControl } from '@jsonforms/core';
import { UISchemaElement } from '@jsonforms/core';
import { ViewTemplate } from '@microsoft/fast-element';

/**
 * Creates a expressions/criteria from given payload
 * @public
 */
export declare const createExpressions: (payload: any) => Expressions;

declare class DispatchRenderer extends FASTElement {
    jsonforms: any;
    jsonformsChanged(): void;
    props: any;
    propsChanged(): void;
    touched: any;
    submitted: boolean;
    submittedChanged(): void;
    dispatch: any;
    control: any;
    renderer: any;
    prefix: string;
    createRenderer: () => void;
    onBlur(): void;
    get determinedRenderer(): unknown;
    disconnectedCallback(): void;
}

declare type Expressions = {
    options: Expression[];
    numberOptions: Expression[];
    dateOptions: Expression[];
};

/**
 * Foundation filters component for automatically generated filters based on json schema
 * obtained from the api, supplied initial data or supplied JSON schema.
 * Allowing customisable filters elements using UI schema and set of custom renderers
 * @beta
 */
export declare class Filters extends FoundationElement {
    /**
     * Name of the backend resource which will provide metadata
     * used to generate filters
     * @public
     */
    resourceName: string;
    private resourceNameChanged;
    clearFiltersData(): void;
    /**
     * UI schema used to define configuration of the layout and elements in the filters
     * Check {@link UiSchemaElement} for possible options
     *
     * @remarks
     * If not provided will be autogenerated based on json schema or initial data
     *
     * @example
     * Here's a simple example:
     * ```
     * const sampleUISchema = {
     *   type: 'VerticalLayout',
     *   elements: [
     *     {
     *       type: 'Control',
     *       scope: '#/properties/QUANTITY',
     *       label: 'Quantity',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/INSTRUMENT_ID',
     *       options: {
     *         allOptionsResourceName: 'INSTRUMENT',
     *         valueField: 'INSTRUMENT_ID',
     *         labelField: 'INSTRUMENT_ID',
     *       },
     *       label: 'Instrument',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/SIDE',
     *       label: 'Side',
     *     },
     *   ],
     * };
     * ```
     * @public
     */
    uischema: UISchemaElement;
    /**
     * Allows to provide set of renderers used by the filters. If not provided it will default to text-field inputs
     * @public
     */
    renderers: RendererEntry[];
    /**
     * Alternatively to providing {@link Form.resourceName} you can hardcode the JSON schema on the client.
     * @public
     * @remarks
     * Use this when you want to avoid fetching metadata from the server but
     * be aware that it could get out of sync if metadata changes on the server
     */
    jsonSchema: JSONSchema7;
    private connect;
    /**
     * Initial data for the filters
     * @public
     */
    data: any;
    /**
     * Created criteria based on the given data that can be used to filter the data
     * @public
     */
    value: string;
    valueChanged(): void;
    /**
     * @internal
     */
    searchFilters(): void;
    /**
     * @internal
     */
    onChange(event: CustomEvent): void;
    disconnectedCallback(): void;
}

/** @internal */
export declare const filtersLogger: Logger;

/** @public */
export declare const filtersRenderers: any[];

/**
 * Foundation form component for automatically generated forms based on json schema
 * obtained from the api, supplied initial data or supplied JSON schema.
 * Allowing customisable form elements using UI schema and set of custom renderers
 * @beta
 */
export declare class Form extends FoundationElement {
    /**
     * Name of the backend resource which will provide metadata
     * used to generate form and later used for submitting data
     * @public
     */
    resourceName: string;
    private resourceNameChanged;
    private removeDataProperties;
    /**
     * UI schema used to define configuration of the layout and elements in the form
     * Check {@link UiSchemaElement} for possible options
     *
     * @remarks
     * If not provided will be autogenerated based on json schema or initial data
     *
     * @example
     * Here's a simple example:
     * ```
     * const sampleUISchema = {
     *   type: 'VerticalLayout',
     *   elements: [
     *     {
     *       type: 'Control',
     *       scope: '#/properties/QUANTITY',
     *       label: 'Quantity',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/INSTRUMENT_ID',
     *       options: {
     *         allOptionsResourceName: 'INSTRUMENT',
     *         valueField: 'INSTRUMENT_ID',
     *         labelField: 'INSTRUMENT_ID',
     *       },
     *       label: 'Instrument',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/SIDE',
     *       label: 'Side',
     *     },
     *   ],
     * };
     * ```
     * @public
     */
    uischema: UISchemaElement;
    /**
     * Allows to provide set of renderers used by the form. If not provided it will default to text-field inputs
     * @public
     */
    renderers: RendererEntry[];
    /**
     * Alternatively to providing {@link Form.resourceName} you can hardcode the JSON schema on the client.
     * @public
     * @remarks
     * Use this when you want to avoid fetching metadata from the server but
     * be aware that it could get out of sync if metadata changes on the server
     */
    jsonSchema: JSONSchema7;
    private errors;
    private connect;
    /**
     * Initial data for the form
     * @public
     */
    data: any;
    /**
     * @internal
     */
    submitting: boolean;
    /**
     * @internal
     */
    submitted: boolean;
    readonly: boolean;
    /**
     * @internal
     */
    _submit(): Promise<void>;
    /**
     * @internal
     */
    onChange(event: CustomEvent): void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Reset the form state
     * @public
     */
    reset(): void;
}

/** @internal */
export declare const foundationFormStyles: ElementStyles;

/** @internal */
export declare const foundationFormTemplate: ViewTemplate<Form, any>;

/** @public */
export declare const genesisErrorTranslator: ErrorTranslator;

/** @internal */
export declare const logger: Logger;

/**
 * Ensures the value of the current field (the one the validator is on) matches the one of the target field.
 * @param target - any other field
 * @returns a validator function that will be run by the smart form during the client-side validation stage
 * @public
 */
export declare const mustMatch: (target: string) => (data: any, path: any, label: any) => {
    instancePath: string;
    message: string;
    schemaPath: string;
    keyword: string;
    params: {};
}[];

/** @public */
export declare type RendererEntry = {
    renderer: ViewTemplate<DispatchRenderer, any>;
    tester: RankedTester;
    mapper: (state: JsonFormsState, ownProps: OwnPropsOfControl) => StatePropsOfControl;
};

/** @public */
export declare const renderers: any[];

/** @public */
export declare type UiSchema = {
    type: UiSchemaElementType;
    elements: UiSchemaElement[];
    dsPrefix?: string;
};

/** @public */
export declare type UiSchemaElement = {
    type: UiSchemaElementType;
    scope: string;
    label?: string;
    placeholder?: string;
    options?: UiSchemaElementOptions;
};

/** @public */
export declare type UiSchemaElementOptions = {
    async?: boolean;
    allOptionsResourceName?: string;
    datasourceConfig?: DatasourceOptions;
    hidden?: boolean;
    isPassword?: boolean;
    validateFn?: (data: any, path: string, label: string) => ErrorObject[];
    valueField?: string;
    labelField?: string;
    textarea?: boolean;
    data?: any[];
    childUiSchema?: UiSchema;
    childElements?: UiSchema[];
    i18n?: {
        [key: string]: any;
    };
};

/** @public */
export declare type UiSchemaElementType = 'Control' | 'VerticalLayout' | any;

export { }
